module Benchmark.Fixture exposing
    ( Model
    , Msg(..)
    , encodeMsg
    , init
    , modelWithDenseList
    , modelWithList
    , msgDecoder
    , msgFromInt
    , msgs
    , update
    )

import AMR.WithDenseList as WithDenseList
import AMR.WithList as WithList
import DenseList
import Json.Decode
import Json.Encode



-- Program


type Msg
    = Increment
    | Decrement


type alias Model =
    { count : Int
    }


msgs : List Msg
msgs =
    List.map msgFromInt (List.range 0 99)


init : ( Model, Cmd Msg )
init =
    ( { count = 0 }, Cmd.none )


update : Msg -> Model -> ( Model, Cmd Msg )
update msg model =
    case msg of
        Increment ->
            ( { model | count = model.count + 1 }, Cmd.none )

        Decrement ->
            ( { model | count = model.count - 1 }, Cmd.none )


encodeMsg : Msg -> Json.Encode.Value
encodeMsg msg =
    case msg of
        Increment ->
            Json.Encode.string "Increment"

        Decrement ->
            Json.Encode.string "Decrement"


msgFromInt : Int -> Msg
msgFromInt n =
    if modBy 2 n == 0 then
        Increment

    else
        Decrement


msgDecoder : Json.Decode.Decoder Msg
msgDecoder =
    Json.Decode.andThen
        (\text ->
            case text of
                "Increment" ->
                    Json.Decode.succeed Increment

                "Decrement" ->
                    Json.Decode.succeed Decrement

                _ ->
                    Json.Decode.fail ("'" ++ text ++ "' is not a valid message")
        )
        Json.Decode.string



-- AMR


modelWithList : WithList.Model Model Msg
modelWithList =
    { appModel = Tuple.first init
    , appMsgs = msgs
    }


modelWithDenseList : WithDenseList.Model Model
modelWithDenseList =
    { appModel = Tuple.first init
    , appMsgs =
        List.foldl
            (\msg ->
                DenseList.insert
                    (Json.Encode.encode 0
                        (encodeMsg msg)
                    )
            )
            DenseList.empty
            msgs
    }
